package splg.examples.batchmultigrouper;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import splg.data.falldaten.Falldaten;
import splg.data.falldaten.FalldatenBehandlung;
import splg.data.falldaten.FalldatenDiagnose;
import splg.data.fallzuteilung.Bereich;
import splg.grouper.Grouper;
import splg.output.grouper.GrouperOutput;

public class BatchMultigrouperExample {
    
    // Aufbau Testdaten:
    // 1 Zeile = 1 Fall
    // burnr;fallid;statistikfall;behandlungart;tarifsystem;alterjahre;altertage;ssw;ggw;dmb;freiwilligkeit;diagnosen;behandlungen
    // diagnosen = hauptdiagnose,diagnose2,diagnose3,...
    // behandlungen = behandlung1,behandlung2,behandlung3,...
    // Alle weiteren variablen werden in diesem Beispiel nicht unterstützt, 
    // ebensowenig z. B. Seitigkeit bei Diagnosen und Behandlungen

    public static final String[] TESTDATA = new String[]{
        // Akutsomatik
        "71200000;1000000;A;3;1;40;0;;;;;S663,W499,R458,F100;",
        "71200000;1000001;A;3;1;30;0;;;;;N440;625,6352",
        // Psychiatrie
        "71200001;2000000;A;3;5;22;0;;;;;F200,F845;94A110,94A120,94A130,94A142,94A150,04A162,94A170,94A183,94A193,94A1A1,94A1B0,94A1C0",
        "71200001;2000001;A;3;5;33;0;;;;;F252,I688,F630,F552,F131,I1000,F172,E6600,E785,M311;",
        // Rehabiliation
        "71200002;3000000;A;3;3;50;0;;;;;I208,Z508,Z540,I2513,I2514,I350,J90,I482,M6250,I1090,E788,N19,E038,"
        +"N390,B952,U5021,U5111,S501,X599;BA5,AA1313,AA1313,AA1322,AA1323,AA1342,AA1342,AA1333,AA1333,"
        +"AA1383,AA1383,AA13A3,AA13A3,AA1393,AA1393,AA1354,A1354,AA1362,AA1363,AA1371,AA1371,AA13B3,"
        +"AA13B3,AA13C3,AA13C3,AA13D3,AA13D3,AA13E2,399520,AA13E2,AA13F2,AA13F2,399520,399520,399520,"
        +"399520,399520,399520,399520,399520,8749,938909"
    };
    

    // Hauptprogramm, parst die Testdaten in Falldaten und
    // gruppiert die Fälle mit der jeweils passenden SPLG-
    // Definition gemäss Tarifsystem.

    public static void main(String[] args) throws Exception {
        // Daten parsen
        List<Falldaten> daten = new ArrayList<>();
        for (String line : TESTDATA) {
            daten.add(parseFalldaten(line));
        }

        // Daten gruppieren
        multigroup(2019, daten);
    }
    
    private static Falldaten parseFalldaten(String line) {
        Falldaten result = new Falldaten();
        String[] fields = line.split(";", -1);
        result.burnr = fields[0];
        result.fallid = fields[1];
        result.statistikfall = fields[2];
        result.behandlungsart = fields[3];
        result.tarifsystem = fields[4];
        result.agey = fields[5];
        result.aged = fields[6];
        result.ssw = fields[7];
        result.ggw = fields[8];
        result.dmb = fields[9];
        result.freiwilligkeit = fields[10];
        if (!fields[11].isBlank()) {
            int idx = 0;
            for (String diagnose : fields[11].split(",")) {
                FalldatenDiagnose d = new FalldatenDiagnose();
                d.rang = idx;
                d.code = diagnose;
                result.diagnosen.add(d);
            }
        }
        if (!fields[12].isBlank()) {
            int idx = 0;
            for (String behandlung : fields[12].split(",")) {
                FalldatenBehandlung b = new FalldatenBehandlung();
                b.rang = idx;
                b.code = behandlung;
                result.behandlungen.add(b);
            }
        }
        return result;
    }

    private static void multigroup(int datenjahr, List<Falldaten> daten) throws Exception {
        // Wir gruppieren im Batchbetrieb eine Reihe von Falldaten.
        // Dabei erzeugen wir on demand für den Versorgungsbereich des
        // Falls eine Grouperinstanz. Wenn es nur Akutfälle sind, wird
        // somit nur ein Akutgrouper erzeugt. Wenn es z. B. Akut- und Reha-
        // fälle sind, werden sowohl ein Akut- als auch ein Rehagrouper
        // erzeugt. Für dieses Testprogramm werden die Daten immer mit dem
        // gegebenen Datenjahr erzeugt. Ausserdem werden immer Grouper-Definitionen
        // mit Suffix "_spl23" verwendet (weil wir solche für alle Versorgungsbereiche
        // vorliegen haben - die definitiven Grouper für 2023 und alle Bereiche
        // können erst im Herbst erstellt werden.)

        Map<Bereich, Grouper> groupers = new HashMap<>();
        try {
            for (Falldaten fall : daten) {
                Bereich bereich = getBereich(fall);
                if (!groupers.containsKey(bereich)) {
                    groupers.put(bereich, createGrouper(datenjahr, bereich));
                }
                Grouper grouper = groupers.get(bereich);
                GrouperOutput output = grouper.group(fall);

                // Der Einfachheit halber geben wir lediglich die zugewiesene SPLG 
                // zusammen mit den Fall-identifizierende Infos aus.
                System.out.println(bereich + " " + fall.burnr + " " + fall.fallid + ": " + output.getSplg());
            }
        } finally {
            for (Grouper grouper : groupers.values()) {
                grouper.close();
            }
        }
    }

    private static Bereich getBereich(Falldaten fall) {
        // Wir bestimmen den Versorgungsbereich anhand des Tarifsystems.
        // Es sind auch andere oder kompliziertere Regeln denkbar.
        switch (fall.tarifsystem) {
            case "1":
            case "2":
                return Bereich.Akutsomatik;

            case "3":
            case "4":
                return Bereich.Rehabilitation;

            case "5":
            case "6":
                return Bereich.Psychiatrie;

            default:
                throw new RuntimeException("Cannot handle tarifsystem " + fall.tarifsystem);
        }
    }

    private static File getBaseDir() {
        File base = new File("07_Definitions/Release"); // internal file, ignore outside of GD
        if (base.exists()) return base;
        return new File("4_lib");
    }

    private static String getBereichDir(Bereich bereich) {
        switch (bereich) {
            case Akutsomatik:
                return "Akut";
            case Psychiatrie:
                return "Psy";
            case Rehabilitation:
                return "Reha";
            default:
                throw new RuntimeException("unknown bereich " + bereich);
        }
    }

    private static Grouper createGrouper(int datenjahr, Bereich bereich) throws Exception {
        File dir = new File(new File(getBaseDir(), getBereichDir(bereich)), datenjahr + "_spl23"); // Suffix _spl23 for demo purposes
        return Grouper.create(bereich, dir);
    }

}
